@@ -430,6 +430,7 @@ bool Parameters::loadFromArgs(int argc, char *argv[])
                 if (code > 0)
                 {
 // convert text to utfOut using iconv because its easier to debug string placements
+                    assert(pText32);
                     pText32[charLength++] = code;
                     if (charLength % 10 == 0)
                         fprintf(log, "%4x\n",code);
@@ -463,7 +464,7 @@ bool Parameters::loadFromArgs(int argc, char *argv[])
             charLength = convertUtf<gr2::utf8>(pText, pText32);
             if (!pText32)
             {
-            	if (charLength == -1)
+            	if (charLength == -1U)
             		perror("decoding utf-8 data failed");
             	else
             		perror("insufficent memory for text buffer");
@@ -485,6 +486,7 @@ bool Parameters::loadFromArgs(int argc, char *argv[])
         }
         else 
         {
+            assert(pText32);
             pText32[charLength] = 0;
             fprintf(log, "\n");
         }
@@ -554,7 +556,6 @@ gr_feature_val * Parameters::parseFeatures(const gr_face * face) const
     if (features && (pLang = strstr(features, "lang=")))
     {
         pLang += 5;
-        size_t i = 0;
         for (int i = 4; i; --i)
         {
             lang_id <<= 8;
@@ -578,6 +579,7 @@ gr_feature_val * Parameters::parseFeatures(const gr_face * face) const
         {
             case ',':
             case '&':
+                assert(valueText);
                 value = atoi(valueText);
                 if (ref)
                 {
@@ -596,6 +598,7 @@ gr_feature_val * Parameters::parseFeatures(const gr_face * face) const
                 }
                 if (!ref)
                 {
+                    assert(name);
                     feat_id = atoi(name);
                     ref = gr_face_find_fref(face, feat_id);
                 }
@@ -612,6 +615,7 @@ gr_feature_val * Parameters::parseFeatures(const gr_face * face) const
         }
         if (ref)
         {
+            assert(valueText);
             value = atoi(valueText);
             gr_fref_set_feature_value(ref, value, featureList);
             if (feat_id > 0x20000000)
@@ -685,6 +689,7 @@ int Parameters::testFileFont() const
             }
             *ui = 0;
             pSeg = gr_make_seg(sizedFont, face, 0, features ? featureList : NULL, (gr_encform)codesize, pText8, textSrc.getLength(), rtl ? 1 : 0);
+            free(pText8);
         }
         else
             pSeg = gr_make_seg(sizedFont, face, 0, features ? featureList : NULL, textSrc.utfEncodingForm(),
@@ -741,7 +746,7 @@ int Parameters::testFileFont() const
             for (unsigned int j = 0; j < numchar; j++)
             {
                 const gr_char_info *c = gr_seg_cinfo(pSeg, j);
-                fprintf(log, "%d\t%04X\t%d\t%d\t%ld\n", j, gr_cinfo_unicode_char(c), gr_cinfo_before(c), gr_cinfo_after(c), gr_cinfo_base(c));
+                fprintf(log, "%d\t%04X\t%d\t%d\t%d\n", j, gr_cinfo_unicode_char(c), gr_cinfo_before(c), gr_cinfo_after(c), int(gr_cinfo_base(c)));
             }
             free(map);
         }
