@@ -162,7 +162,7 @@ inline void Machine::Code::externalProgramMoved(ptrdiff_t dist) throw()
 {
     if (_code && !_own)
     {
-        _code += dist / sizeof(instr);
+        _code += dist / signed(sizeof(instr));
         _data += dist;
     }
 }
